def seperate_string_number(string):
	previous_character = string[0]
	groups = []
	alsonumeric = [".", "-"]
	newword = string[0]
	for x, i in enumerate(string[1:]):
		if i.isalpha() and previous_character.isalpha():
			newword += i
		elif (i.isnumeric() or i in alsonumeric) and (previous_character.isnumeric() or previous_character in alsonumeric):
			newword += i
		else:
			groups.append(newword)
			newword = i

		previous_character = i

		if x == len(string) - 2:
			groups.append(newword)
			newword = ''
	return groups


from .basics import number, seperate_string_number, DataType

class unit(DataType):
	def __init__(self, j4j, name, default, units={}):
		super().__init__(j4j)
		self.name = name
		self.t = str
		self.default = default
		self.units = units
	#returns true/false, whether or not the data is this type
	def matches(self, data):
		if type(data) == str:
			if seperate_string_number(data)[1] in self.units:
				return True
			else:
				self.j4j.log(f"Error: {self.name} does not recognize \"{seperate_string_number(data)[1]}\" as a valid unit.", level=5)
		return False
	def convert(self, data, rule, skipRuleConversion=False, useUnit=None, parentUID="ROOT"):#useUnit, if not set to none, skips looking through the rule file and uses whatever it's set to as the new unit.
		unitToUse = self.j4j.get_object(parentUID)['_defaults']['unit'][self.name]
		#check if there's a rule for this, that would ovverride unitToUse
		if skipRuleConversion == False:
			if "unit" in rule:
				unitToUse = rule['unit']
		if useUnit != None:
			unitToUse = useUnit
		
		if skipRuleConversion == False:
			alsoConvert = ["min", "max", "multiplier"] #we should also convert these values to preffered measurements
			for x in alsoConvert:
				if x in rule:
					rule[x] = self.convert(rule[x], rule, skipRuleConversion=True, useUnit=unitToUse)
		
		value = seperate_string_number(data) #gets the number and then the units
		valueInDefaultUnits = self.units[value[1]] * float(value[0]) #converts whatever units were used to seconds/meters

		valueInDesiredUnits = valueInDefaultUnits / self.units[unitToUse] #converts to the value we should be using
		numberValue = self.j4j.dataTypes['number'].convert(valueInDesiredUnits, rule, parentUID=parentUID) #do normal number conversions
		return numberValue


class time(unit):
	def __init__(self, ajson):
		super().__init__(ajson, "time", "0s", units={"ms":1/1000, "μs": 1/1000000, "us": 1/1000000, "ns": 1/(1000000000), "":1, "sec": 1, "secs": 1, "s": 1, "m": 60, "min": 60, "mins": 60, "h": 60*60, "d": 60*60*24, "W": 60*60*24*7, "M": 60*60*24*7*30, "Y": 60*60*24*365})

class distance(unit):
	def __init__(self, ajson):
		super().__init__(ajson, "distance", "0m", units={"":1, "me": 1, "m": 1, "km": 1000, "ft": 3.280839895, "mi": 1000 * 0.621371, "cm": .001, "in": 3.280839895/12})



from ..utils import seperate_string_number
import math
from ..JSON4JSON import JSON4JSON

class DataType:
	def __init__(self, j4j:JSON4JSON):
		self.j4j = j4j
		#name, t, and default all need to be set.
		self.name = "none"
		self.t = None
		self.default = None
	#returns true/false, whether or not the data is this type
	def matches(self, data):
		if type(data) == self.t:
			return True
		return False
	#data - the data provided from the config file. rule - the rules for this property.
	def convert(self, data, ruleset, parentUID="ROOT"):
		if type(data) != self.t:
			raise Exception("Invalid type!")
		return data



class string(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "string"
		self.t = str
		self.default = ""
	
	def convert(self, data, rule, parentUID="ROOT"): #strings have no rules YET (todo: add min/max length)
		return str(data)

class number(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "number"
		self.t = float
		self.default = 0
	#returns true/false, whether or not the data is this type
	def matches(self, data):
		newData = data
		try:
			newData = float(data)
		except:
			return False
		return type(newData) == float or type(newData) == int
	
	def convert(self, data, rule, parentUID="ROOT"):
		value = float(data)
		minval = self.j4j.get_property(rule, "min", parentUID, noneFound=-math.inf)
		maxval = self.j4j.get_property(rule, "max", parentUID, noneFound=math.inf)
		multiplier = float(self.j4j.get_property(rule, "multiplier", parentUID, noneFound=1))
		value *= multiplier
		isInt = self.j4j.get_property(rule, "int", parentUID, noneFound=False)
		if isInt:
			value = int(value)
		return min(max(value, minval), maxval)

class anyType(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "any"
		self.default = None
	#returns true/false, whether or not the data is this type
	def matches(self, data):
		return True
	def convert(self, data, rule, parentUID="ROOT"):
		return data

class boolean(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "bool"
		self.t = bool
		self.default = False

	def convert(self, data, rule, parentUID="ROOT"):
		return data == True


class arrayType(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "array"
		self.t = list
		self.default = []

	def convert(self, data, rules, parentUID="ROOT"):
		#length
		minLength = self.j4j.get_property(rules, "minLength", parentUID, noneFound=0)
		maxLength = self.j4j.get_property(rules, "maxLength", parentUID, noneFound=math.inf)
		if len(data) < minLength:
			self.j4j.error(f"Array too short!")
		if len(data) > maxLength:
			data = data[:maxLength]
		if 'rule' not in rules:
			defType = self.j4j.get_property(rules, "rule", parentUID, noneFound=self.j4j.get_object(parentUID)['_defaults']['t'])
			if defType == "array": defType = "string"
			rules['rule'] = {"t": defType}
		
		for x,i in zip(data, range(0, len(data))):
			#convert single
			usedRule = rules['rule']
			#check if we have any custom defined rules
			if f"rule#{i}" in rules:
				usedRule = rules[f'rule#{i}']
			data[i] = self.j4j.convert_single(x, usedRule, parentUID=parentUID, name=x)
		return data



import pathlib
from setuptools import _install_setup_requires, setup, find_packages

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
setup(
	name = "JSON4JSON",
	version = "0.4.2",
	description = "Configure JSON with JSON",
	long_description = README,
	long_description_content_type = "text/markdown",
	url = "https://github.com/TrevorBagels/JSON4JSON",
	license = "GNU GPLv3",
	classifiers=[
		"License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
		"Programming Language :: Python :: 3",
		"Programming Language :: Python :: 3.7",
    ],
	author = "Trevor Bagels",
	author_email = "trevorbagels@gmail.com",
	packages=find_packages(),
	include_package_data=True
)

#notes for building and releasing
#python3 -m build
#python3 -m twine upload --repository pypi dist/*


from JSON4JSON.DataTypes.basics import DataType
from ..utils import seperate_string_number
import math
from ..JSON4JSON import JSON4JSON

class choice(DataType):
	def __init__(self, j4j):
		super().__init__(j4j)
		self.name = "choice"
		self.t = any

	def matches(self, data):
		#choice is either gonna be a number, a string, or uh, well yeah that's basically it. I think.
		return type(data) not in [dict, list]
	
	def convert(self, data, ruleset, parentUID="ROOT"):
		options = self.j4j.get_property(ruleset, "options",
			parentUID=parentUID,
			noneFound=self.j4j.get_parent(parentUID)['_defaults']['options'])
		if data not in options:
			self.j4j.error(f"\"{data}\" is not a valid option. It should be one of the following: {', '.join(options)}")
		else:
			return data
		

	